#include <iostream>
#include <vector>
#include <algorithm>
#include <cassert>
#include <fstream>
#include <set>

using namespace std;

static const int kModulo = 1000 * 1000 * 1000 + 7;

bool unique(vector<int> V) {
    sort(V.begin(), V.end());
    for (int i = 0; i < int(V.size()); ++i)
        if (i > 0 && V[i] != 0 && V[i] == V[i - 1])
            return false;
    return true;
}

vector<int> have(vector<int> V) {
    V.erase(remove(V.begin(), V.end(), 0), V.end());
    return V;
}

vector<int> not_have(vector<int> V) {
    vector<int> answer(V.size());
    for (int i = 0; i < int(answer.size()); ++i)
        answer[i] = i + 1;
    answer.erase(remove_if(answer.begin(), answer.end(), [&](int x) {
        return count(V.begin(), V.end(), x) > 0;
    }), answer.end());
    return answer;
}

void normalize(vector<int>& up, vector<int>& down) {
    set<int> have_up(up.begin(), up.end());
    set<int> have_down(down.begin(), down.end());

    up.clear();
    down.clear();
    int maxv = max(*have_up.rbegin(), *have_down.rbegin());

    up.push_back(0);
    down.push_back(0);
    for (int i = 1; i <= maxv; ++i) {
        if (have_up.count(i)) {
            up.push_back(up.back() + 1);
            down.push_back(down.back());
        }
        if (have_down.count(i)) {
            up.push_back(up.back());
            down.push_back(down.back() + 1);
        }
    }
}

vector<int> solve(vector<int> up, vector<int> down) {
    assert(up.size() == down.size());
    int N = up.size();
    if (up.empty())
        return vector<int>(1, 1);
    normalize(up, down);

    vector< vector<int> > dp(N + 1, vector<int>(N + 1, 0));
    dp[0][0] = 1;

    for (int i = 0; i < 2 * N; ++i) {
        vector< vector<int> > next_dp(N + 1, vector<int>(N + 1, 0));
        for (int won = 0; won <= N; ++won)
            for (int lost = 0; lost <= N; ++lost) {
                bool is_up = (up[i + 1] - up[i]) != 0;
                int unmatched_up = up[i] - won - lost;
                int unmatched_down = down[i] - won - lost;
                next_dp[won][lost] = (next_dp[won][lost] + dp[won][lost]) % kModulo;
                if (is_up && won < N && unmatched_down > 0)
                    next_dp[won + 1][lost] = (next_dp[won + 1][lost] + 1LL * dp[won][lost] * unmatched_down) % kModulo;
                if (!is_up && lost < N && unmatched_up > 0)
                    next_dp[won][lost + 1] = (next_dp[won][lost + 1] + 1LL * dp[won][lost] * unmatched_up) % kModulo;
            }
        swap(dp, next_dp);
    }
    vector<int> ways(N + 1, 0);
    for (int won = 0; won <= N; ++won) {
        int lost = N - won;
        ways[won] = (ways[won] + 1LL * dp[won][lost]) % kModulo;
    }

    return ways;
}

int main() {
    ifstream cin("pitmutare.in");
    ofstream cout("pitmutare.out");

    int N, S; assert(cin >> N >> S);
    assert(1 <= N && N <= 300);
    assert(0 <= S && S < N);
    vector<int> A(N);
    for (int i = 0; i < N; ++i) {
        assert(cin >> A[i]);
        assert(0 <= A[i] && A[i] <= N);
    }

    vector<int> B(N);
    for (int i = 0; i < N; ++i) {
        assert(cin >> B[i]);
        assert(0 <= B[i] && B[i] <= N);
    }
    for (int i = 0; i < N; ++i) {
        assert(A[i] == 0 || B[i] == 0);
        assert(A[i] > 0 || B[i] > 0);
    }
    assert(unique(A));
    assert(unique(B));
    auto first = solve(have(A), not_have(B));
    auto second = solve(not_have(A), have(B));

    int answer = 0;
    for (int i = 0; i <= S; ++i)
        if (i < int(first.size()) && S - i < int(second.size()))
            answer = (answer + int64_t(first[i]) * second[S - i]) % kModulo;
    cout << answer << "\n";
}
