// O(L*N^3 + Q*N^2*logL)

#include <bits/stdc++.h>

using namespace std;

#define mp make_pair
#define pb push_back
#define ll long long

const int QMAX = 3 * 1e5;
const int LMAX = 3 * 1e4;
const int NMAX = 28;
const int VALMAX = 1e4;

const int lmax = 3 * 1e4 + 5;
const int nmax = NMAX + 1;
const int inf = (1 << 29);

struct Edge {
	int x, y, c, r;
	Edge() {}
	Edge(int x, int y, int c, int r) {
		this->x = x;
		this->y = y;
		this->c = c;
		this->r = r;
	}
};

struct Matrix {
	int cost[nmax][nmax];
	int Nn;
	Matrix() {}
	Matrix(int n2) {
		this->Nn = n2;
		assert(this->Nn >= 2 && this->Nn <= NMAX);
		for (int i = 1; i <= this->Nn; ++i) {
			for (int j = 1; j <= this->Nn; ++j) {
				this->cost[i][j] = inf;
			}
		}
	}

	Matrix(int n2, Edge &edge) : Matrix(n2) {
		for (int i = 1; i <= this->Nn; ++i) {
			this->cost[i][i] = edge.r;
		}
		// this->cost[edge.x][edge.y] = edge.c;
		// this->cost[edge.y][edge.x] = edge.c;
		update(this->cost[edge.x][edge.y], edge.c);
		update(this->cost[edge.y][edge.x], edge.c);
	}

	Matrix(Matrix &other) {
		this->Nn = other.Nn;
		assert(this->Nn >= 2 && this->Nn <= NMAX);
		for (int i = 1; i <= this->Nn; ++i) {
			for (int j = 1; j <= this->Nn; ++j) {
				this->cost[i][j] = other.cost[i][j];
			}
		}
	}

	void update(int &x, int y) {
		x = min(x, y);
	}

	void merge(Matrix &other) {
		// merge self with other;
		Matrix newMatrix(this->Nn);
		for (int i = 1; i <= this->Nn; ++i) {
			for (int j = 1; j <= this->Nn; ++j) {
				for (int k = 1; k <= this->Nn; ++k) {
					if (cost[i][k] != inf && other.cost[k][j] != inf) {
						update(newMatrix.cost[i][j], cost[i][k] + other.cost[k][j]);
					}
				}
			}
		}
		assign(newMatrix);
	}

	void merge(Matrix &other, int &source) {
		// merge self with other;
		Matrix newMatrix(this->Nn);
		for (int j = 1; j <= this->Nn; ++j) {
			for (int k = 1; k <= this->Nn; ++k) {
				if (cost[source][k] != inf && other.cost[k][j] != inf) {
					update(newMatrix.cost[source][j], cost[source][k] + other.cost[k][j]);
				}
			}
		}
		assign(newMatrix);
	}

	void assign(Matrix &other) {
		for (int i = 1; i <= this->Nn; ++i) {
			for (int j = 1; j <= this->Nn; ++j) {
				cost[i][j] = other.cost[i][j];
			}
		}
	}

	void init(int x, int y) {
		cost[x][x] = 0;
		cost[y][y] = 0;
	}

	string toString() {
		string s = "";
		for (int i = 1; i <= this->Nn; ++i) {
			for (int j = 1; j <= this->Nn; ++j) {
				s += (cost[i][j] == inf ? "-1" : to_string(cost[i][j])) + " ";
			}
			s += "\n";
		}
		return s;
	}
};

int n, L, Q;
Edge s[lmax];
Matrix aint[int(lmax * 2.2)];

void build(int node, int st, int dr) {
	if (st == dr) {
		aint[node] = Matrix(n, s[st]);
		return;
	}
	int mij = (st + dr) / 2;
	build(node * 2, st, mij);
	build(node * 2 + 1, mij + 1, dr);
	assert(aint[node * 2].Nn >= 2 && aint[node * 2 + 1].Nn <= NMAX);
	aint[node] = Matrix(aint[node * 2]);
	aint[node].merge(aint[node * 2 + 1]);
}

void query(int node, int st, int dr, int a, int b, Matrix &ans, int &source) {
	if (a <= st && dr <= b) {
		ans.merge(aint[node], source);
		return;
	}
	int mij = (st + dr) / 2;
	if (a <= mij) {
		query(node * 2, st, mij, a, b, ans, source);
	}
	if (b > mij) {
		query(node * 2 + 1, mij + 1, dr, a, b, ans, source);
	}
}

int main() {
	ios::sync_with_stdio(false);
#ifndef ONLINE_JUDGE
	freopen("razbunare.in", "r", stdin);
	freopen("razbunare.out", "w", stdout);
#endif

	cin >> n >> L >> Q;
	assert(n >= 2 && n <= NMAX);
	assert(L >= 1 && L <= LMAX);
	assert(Q >= 1 && Q <= QMAX);

	for (int i = 1; i <= L; ++i) {
		int x, y, c, r;
		cin >> x >> y >> c >> r;
		assert(x >= 1 && x <= n);
		assert(y >= 1 && y <= n);
		// assert(x != y);
		assert(c >= 0 && c <= VALMAX);
		assert(r >= 0 && r <= VALMAX);
		s[i] = Edge(x, y, c, r);
	}

	build(1, 1, L);
	for (int i = 1; i <= Q; ++i) {
		int x, y, a, b;
		cin >> x >> y >> a >> b;
		assert(1 <= x && x <= n);
		assert(1 <= y && y <= n);
		assert(1 <= a && a <= L);
		assert(1 <= b && b <= L);
		assert(a <= b);
		Matrix ans(n);
		ans.init(x, y);
		query(1, 1, L, a, b, ans, x);
		// swap(x, y);
		int ras = (ans.cost[x][y] == inf ? -1 : ans.cost[x][y]);
		assert(ras >= -1);
		cout << ras << "\n";
	}

	return 0;
}